﻿using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Effects;
using System.Reflection;
using System.IO;

namespace DynamicShader
{
    public class CustomEffect : ShaderEffect
    {
        #region Constructors

        public CustomEffect(byte[] bytecode)
        {
            var shader = new PixelShader();
            shader.SetStreamSource(new MemoryStream(bytecode));
            this.PixelShader = shader;

            UpdateShaderValue(InputProperty);
            UpdateShaderValue(ElapsedSecondsProperty);
        }

        #endregion

        #region Dependency Properties

        public Brush Input
        {
            get { return (Brush)GetValue(InputProperty); }
            set { SetValue(InputProperty, value); }
        }

        // Brush-valued properties turn into sampler-property in the shader.
        // This helper sets "ImplicitInput" as the default, meaning the default
        // sampler is whatever the rendering of the element it's being applied to is.
        public static readonly DependencyProperty InputProperty =
            ShaderEffect.RegisterPixelShaderSamplerProperty("Input", typeof(CustomEffect), 0);

        public double ElapsedSeconds
        {
            get { return (double)GetValue(ElapsedSecondsProperty); }
            set { SetValue(ElapsedSecondsProperty, value); }
        }

        public static readonly DependencyProperty ElapsedSecondsProperty =
            DependencyProperty.Register("ElapsedSeconds", typeof(double), typeof(CustomEffect),
                    new UIPropertyMetadata(0.0, PixelShaderConstantCallback(0)));

        #endregion

        #region Member Data

        #endregion

    }
}
