﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Threading;

namespace DynamicShader
{
    using D3DX;

    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        private DispatcherTimer _timer = new DispatcherTimer();

        public Window1()
        {
            InitializeComponent();

            this.Loaded += new RoutedEventHandler(Window1_Loaded);
        }

        void Window1_Loaded(object sender, RoutedEventArgs e)
        {
            this.mediaElement.LoadedBehavior = MediaState.Manual;
            this.mediaElement.UnloadedBehavior = MediaState.Manual;
            this.mediaElement.MediaOpened += (_, __) =>
            {
                if (this.mediaElement.NaturalDuration.HasTimeSpan)
                {
                    timelineProgressBar.Maximum = this.mediaElement.NaturalDuration.TimeSpan.TotalSeconds;
                }
            }; 
            this.mediaElement.MediaEnded += (_, __) =>
            {
                if (!this.mediaElement.NaturalDuration.HasTimeSpan)
                {
                    timelineProgressBar.Maximum = this.mediaElement.Position.TotalSeconds;
                }
                this.mediaElement.Position = new TimeSpan();
            };

            const string initialCode =
@"sampler2D implicitInput : register(s0);
float elapsedSeconds : register(c0);

float4 main(float2 uv : TEXCOORD) : COLOR
{
    float phase = 10.0f*(uv.y + elapsedSeconds);
    float2 offset = 0.1f * float2(cos(phase), 0.0f);
    return tex2D(implicitInput, uv + offset);
}
";
            this.codingTextBox.Text = initialCode;

            this.resetButton.Click += (_, __) =>
                {
                    this.codingTextBox.Text = initialCode;
                };

            this.compileButton.Click += (_, __) =>
            {
                var effect = CompiledEffect.FromString(this.codingTextBox.Text, CompilerOptions.None);
                if(!effect.CompileSucceeded)
                {
                    this.errorShowTextBox.Text = effect.ErrorMessage;
                }
                else
                {
                    var shader = effect.CompileShader("main", "ps_2_0", CompilerOptions.None);
                    if(!shader.CompileSucceeded)
                    {
                        this.errorShowTextBox.Text = shader.ErrorMessage;
                    }
                    else
                    {
                        this.errorShowTextBox.Text = "Success";
                        var customEffect = new CustomEffect(shader.Data);
                        var valueBinding = new Binding("Value")
                        {
                            Source = this.timelineProgressBar,
                        };
                        BindingOperations.SetBinding(customEffect, CustomEffect.ElapsedSecondsProperty, valueBinding);
                        if (this.mediaElement.Effect != null)
                        {
                            BindingOperations.ClearAllBindings(this.mediaElement.Effect);
                        }

                        this.mediaElement.Effect = customEffect;
                    }
                }
            };

            _timer.Interval = new TimeSpan(0, 0, 0, 0, 33);
            _timer.Tick += (_, __) =>
            {
                this.timelineProgressBar.Value = this.mediaElement.Position.TotalSeconds;
            };
            _timer.Start();

            this.mediaOpenButton.Click += (_, __) =>
            {
                var dialog = new Microsoft.Win32.OpenFileDialog()
                {
                    Multiselect = false,
                };

                if (dialog.ShowDialog() == true)
                {
                    this.mediaElement.Source = new Uri(dialog.FileName, UriKind.RelativeOrAbsolute);
                    this.mediaElement.Play();
                }
            };
        }
    }
}
